<?php

/*
 * Plugin Name: Woo Infinite Options
 * Version: 1.114
 * Plugin URI: https://www.persianscript.ir/
 * Description: This unique plugin allows you to easily configure smart options for your Woocommerce products.
 * Author: persianscript.ir/
 * Author URI: https://www.persianscript.ir/
 * Requires at least: 5.7
 * Tested up to: 5.9.3
 *
 * @package WordPress
 * @author Biscay Charly (loopus)
 * @since 1.0.0
 */

if (!defined('ABSPATH'))
    exit;

register_activation_hook(__FILE__, 'wio_install');
register_uninstall_hook(__FILE__, 'wio_uninstall');

global $jal_db_version;
$jal_db_version = "1.1";

require_once('includes/wio_core.php');
require_once('includes/wio_admin.php');

function wio_WooInfiniteOptions() {
    $version = 1.114;
    $instance = wio_core::instance(__FILE__, $version);
    if (is_null($instance->menu)) {
        $instance->menu = wio_admin::instance($instance);
    }
    return $instance;
}

/**
 * Installation. Runs on activation.
 * @access  public
 * @since   1.0.0
 * @return  void
 */
function wio_install() {
    global $wpdb;
    global $jal_db_version;
    require_once(ABSPATH . '/wp-admin/includes/upgrade.php');
    add_option("jal_db_version", $jal_db_version);

    $db_table_name = $wpdb->prefix . "wio_settings";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";

        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
    		id mediumint(9) NOT NULL AUTO_INCREMENT,
    		purchaseCode VARCHAR(250) NOT NULL,   
                firstStart BOOL NOT NULL,
		UNIQUE KEY id (id)
		) $charset_collate;";

        dbDelta($sql);
        $rows_affected = $wpdb->insert($db_table_name, array('purchaseCode' => ''));
    }
    
    $db_table_name = $wpdb->prefix . "wio_customizers";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";

        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
    		id mediumint(9) NOT NULL AUTO_INCREMENT,
                jsonData LONGTEXT,
		UNIQUE KEY id (id)
		) $charset_collate;";

        dbDelta($sql);
    }
}

/**
 * Uninstallation.
 * @access  public
 * @since   1.0.0
 * @return  void
 */
function wio_uninstall() {
    global $wpdb;
    global $jal_db_version;
    $table_name = $wpdb->prefix . "wio_settings";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
    $table_name = $wpdb->prefix . "wio_customizers";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
}

wio_WooInfiniteOptions();
