<?php

if (!defined('ABSPATH'))
    exit;

class wio_admin {

    /**
     * The single instance
     * @var    object
     * @access  private
     * @since    1.0.0
     */
    private static $_instance = null;

    /**
     * The main plugin object.
     * @var    object
     * @access  public
     * @since    1.0.0
     */
    public $parent = null;

    /**
     * Prefix for plugin settings.
     * @var     string
     * @access  public
     * Delete
     * @since   1.0.0
     */
    public $base = '';

    /**
     * Available settings for plugin.
     * @var     array
     * @access  public
     * @since   1.0.0
     */
    public $settings = array();

    public function __construct($parent) {
        $this->parent = $parent;
        $this->dir = dirname($this->parent->file);
        add_action('admin_menu', array($this, 'add_menu_item'));
        add_action('admin_init', array($this, 'checkActions'));
        add_action('admin_init', array($this, 'checkAutomaticUpdates'));
        add_action('admin_init', array($this, 'checkFirstStart'));

        add_action('admin_print_scripts', array($this, 'load_admin_scripts'));
        add_action('admin_print_styles', array($this, 'load_admin_styles'));

        add_action('woocommerce_product_data_panels', array($this, 'display_product_edition_fields'));
        add_filter('woocommerce_product_data_tabs', array($this, 'create_product_edition_tab'));
        add_action('woocommerce_process_product_meta', array($this, 'save_product_edition_fields'));

        add_action('wp_ajax_nopriv_wio_loadCustomizers', array($this, 'loadCustomizers'));
        add_action('wp_ajax_wio_loadCustomizers', array($this, 'loadCustomizers'));
        add_action('wp_ajax_nopriv_wio_loadCustomizer', array($this, 'loadCustomizer'));
        add_action('wp_ajax_wio_loadCustomizer', array($this, 'loadCustomizer'));
        add_action('wp_ajax_nopriv_wio_createCustomizer', array($this, 'createCustomizer'));
        add_action('wp_ajax_wio_createCustomizer', array($this, 'createCustomizer'));
        add_action('wp_ajax_nopriv_wio_saveCustomizer', array($this, 'saveCustomizer'));
        add_action('wp_ajax_wio_saveCustomizer', array($this, 'saveCustomizer'));
        add_action('wp_ajax_nopriv_wio_removeCustomizer', array($this, 'removeCustomizer'));
        add_action('wp_ajax_wio_removeCustomizer', array($this, 'removeCustomizer'));
        add_action('wp_ajax_nopriv_wio_exportData', array($this, 'exportData'));
        add_action('wp_ajax_wio_exportData', array($this, 'exportData'));
        add_action('wp_ajax_nopriv_wio_importData', array($this, 'importData'));
        add_action('wp_ajax_wio_importData', array($this, 'importData'));
        add_action('wp_ajax_nopriv_wio_duplicateCustomizer', array($this, 'duplicateCustomizer'));
        add_action('wp_ajax_wio_duplicateCustomizer', array($this, 'duplicateCustomizer'));
        add_action('wp_ajax_nopriv_wio_validatePurchaseCode', array($this, 'validatePurchaseCode'));
        add_action('wp_ajax_wio_validatePurchaseCode', array($this, 'validatePurchaseCode'));
        add_action('wp_ajax_nopriv_wio_previewCustomizer', array($this, 'previewCustomizer'));
        add_action('wp_ajax_wio_previewCustomizer', array($this, 'previewCustomizer'));
        add_action('wp_ajax_nopriv_wio_getWooProductsByTerm', array($this, 'getWooProductsByTerm'));
        add_action('wp_ajax_wio_getWooProductsByTerm', array($this, 'getWooProductsByTerm'));
    }

    public function getWooProductsByTerm() {
        global $wpdb;
        if (current_user_can('manage_options')) {
            $searched = sanitize_text_field($_POST['term']);
            $rep = array();

            if (is_plugin_active('woocommerce/woocommerce.php')) {

                $search_query = 'SELECT ID FROM ' . $wpdb->prefix . 'posts
                  WHERE post_type = "product"
                  AND post_title LIKE %s';

                $like = '%' . $searched . '%';
                $results = $wpdb->get_results($wpdb->prepare($search_query, $like), ARRAY_N);
                $quote_ids = array();
                foreach ($results as $key => $array) {
                    $quote_ids[] = $array[0];
                }

                $products = get_posts(array('post_type' => 'product', 'orderby' => 'category', 'order' => 'ASC', 'post__in' => $quote_ids, 'posts_per_page' => 200));

                foreach ($products as $productI) {

                    $product = wc_get_product($productI->ID);
                    $data = new stdClass();
                    $data->id = $productI->ID;
                    $data->label = $productI->post_title;
                    $rep[] = $data;
                }
            }
            echo json_encode($rep);
        }
        die();
    }

    public function load_admin_styles() {
        if (isset($_GET['post']) && isset($_GET['action']) && $_GET['action'] == 'edit') {
            wp_register_style('bootstrap-5', esc_url($this->parent->assets_url) . 'css/bootstrap.min.css', array(), $this->parent->_version);
            wp_enqueue_style('bootstrap-5');
            wp_register_style('fontawesome', esc_url($this->parent->assets_url) . 'css/fontawesome_all.min.css', array(), $this->parent->_version);
            wp_enqueue_style('fontawesome');
        }
        if ((isset($_GET['page']) && $_GET['page'] == 'wio_menu')) {
            wp_register_style('bootstrap-5', esc_url($this->parent->assets_url) . 'css/bootstrap.min.css', array(), $this->parent->_version);
            wp_enqueue_style('bootstrap-5');
            wp_register_style('jquery-richtext', esc_url($this->parent->assets_url) . 'css/jquery-richtext.min.css', array(), $this->parent->_version);
            wp_enqueue_style('jquery-richtext');
            wp_register_style('fontawesome', esc_url($this->parent->assets_url) . 'css/fontawesome_all.min.css', array(), $this->parent->_version);
            wp_enqueue_style('fontawesome');
            wp_register_style('codemirror', esc_url($this->parent->assets_url) . 'css/codemirror.min.css', array(), $this->parent->_version);
            wp_enqueue_style('codemirror');
            wp_register_style('codemirror-xq-light', esc_url($this->parent->assets_url) . 'css/codemirror-xq-light.min.css', array(), $this->parent->_version);
            wp_enqueue_style('codemirror-xq-light');
            wp_enqueue_style('thickbox');

            wp_register_style($this->parent->_token . '_dark', esc_url($this->parent->assets_url) . 'css/wio_dark.min.css', array(), $this->parent->_version);
            wp_enqueue_style($this->parent->_token . '_dark');
            wp_register_style($this->parent->_token . '_admin', esc_url($this->parent->assets_url) . 'css/wio_admin.min.css', array(), $this->parent->_version);
            wp_enqueue_style($this->parent->_token . '_admin');
        }
    }

    public function load_admin_scripts() {
        if (isset($_GET['post']) && isset($_GET['action']) && $_GET['action'] == 'edit') {

            wp_register_script('bootstrap-5', esc_url($this->parent->assets_url) . 'js/bootstrap.bundle.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('bootstrap-5');

            wp_register_script($this->parent->_token . '_productEdition', esc_url($this->parent->assets_url) . 'js/wio_productEdition.min.js', array('bootstrap-5', 'jquery'), $this->parent->_version, true);
            wp_enqueue_script($this->parent->_token . '_productEdition');

            $js_data[] = array(
                'assetsUrl' => esc_url($this->parent->assets_url),
                'placeholderImg' => wc_placeholder_img_src(128),
                'texts' => array(
                    'Edit' => esc_html__('Edit', 'wio'),
                    'Remove' => esc_html__('Remove', 'wio'),
                    'Visibility conditions' => esc_html__('Visibility conditions', 'wio')
                )
            );
            wp_localize_script($this->parent->_token . '_productEdition', 'wio_data', $js_data);
        }
        if ((isset($_GET['page']) && $_GET['page'] == 'wio_menu')) {
            wp_register_script('bootstrap-5', esc_url($this->parent->assets_url) . 'js/bootstrap.bundle.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('bootstrap-5');
            wp_register_script('jquery-richtext', esc_url($this->parent->assets_url) . 'js/jquery-richtext.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('jquery-richtext');
            wp_register_script('pixijs', esc_url($this->parent->assets_url) . 'js/pixi.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('pixijs');
            wp_register_script('codemirror', esc_url($this->parent->assets_url) . 'js/codemirror.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('codemirror');
            wp_register_script('codemirror-javascript', esc_url($this->parent->assets_url) . 'js/codemirror-javascript.min.js', array(), $this->parent->_version, true);
            wp_enqueue_script('codemirror-javascript');
            wp_enqueue_script('thickbox');

            wp_register_script($this->parent->_token . '_admin', esc_url($this->parent->assets_url) . 'js/wio_admin.min.js', array('bootstrap-5', 'pixijs', 'jquery', 'jquery-ui-core','jquery-ui-sortable', 'jquery-ui-autocomplete', 'jquery-ui-draggable','jquery-ui-sortable'), $this->parent->_version, true);
            wp_enqueue_script($this->parent->_token . '_admin');

            $js_data[] = array(
                'assetsUrl' => esc_url($this->parent->assets_url),
                'placeholderImg' => wc_placeholder_img_src(128),
                'texts' => array(
                    'My customizer' => esc_html__('My customizer', 'wio'),
                    'My step' => esc_html__('My step', 'wio'),
                    'Edit' => esc_html__('Edit', 'wio'),
                    'Remove' => esc_html__('Remove', 'wio'),
                    'Duplicate' => esc_html__('Duplicate', 'wio'),
                    'My title' => esc_html__('My title', 'wio'),
                    'Title' => esc_html__('Title', 'wio'),
                    'Fields' => esc_html__('Fields', 'wio'),
                    'Datepicker' => esc_html__('Datepicker', 'wio'),
                    'Number field' => esc_html__('Number field', 'wio'),
                    'Text field' => esc_html__('Text field', 'wio'),
                    'File upload' => esc_html__('File upload', 'wio'),
                    'Checkbox' => esc_html__('Checkbox', 'wio'),
                    'Radio' => esc_html__('Radio', 'wio'),
                    'Color' => esc_html__('Color', 'wio'),
                    'File field' => esc_html__('File field', 'wio'),
                    'Button' => esc_html__('Button', 'wio'),
                    'Select menu' => esc_html__('Select menu', 'wio'),
                    'Add a condition' => esc_html__('Add a condition', 'wio'),
                    'Delete the group' => esc_html__('Delete the group', 'wio'),
                    'Value' => esc_html__('Value', 'wio'),
                    'Price' => esc_html__('Price', 'wio'),
                    'Text' => esc_html__('Text', 'wio'),
                    'Number' => esc_html__('Number', 'wio'),
                    'And' => esc_html__('And', 'wio'),
                    'Or' => esc_html__('Or', 'wio'),
                    'Type' => esc_html__('Type', 'wio'),
                    'Step' => esc_html__('Step', 'wio'),
                    'Field' => esc_html__('Field', 'wio'),
                    'Operator' => esc_html__('Operator', 'wio'),
                    'Value' => esc_html__('Value', 'wio'),
                    'Is filled' => esc_html__('Is filled', 'wio'),
                    'Isn\'t filled' => esc_html__('Isn\'t filled', 'wio'),
                    'Is equal to' => esc_html__('Is equal to', 'wio'),
                    'Is different than' => esc_html__('Is different than', 'wio'),
                    'Is superior to' => esc_html__('Is superior to', 'wio'),
                    'Is inferior to' => esc_html__('Is inferior to', 'wio'),
                    'Customize it !' => esc_html__('Customize it !', 'wio'),
                    'Is selected' => esc_html__('Is selected', 'wio'),
                    "Isn't selected" => esc_html__("Isn't selected", 'wio'),
                    'Continue' => esc_html__('Continue', 'wio'),
                    'Option' => esc_html__('Option', 'wio'),
                    'Value' => esc_html__('Value', 'wio'),
                    'Total' => esc_html__('Total', 'wio'),
                    'Selected options' => esc_html__('Selected options', 'wio'),
                    'Text area' => esc_html__('Text area', 'wio'),
                    'Checkbox' => esc_html__('Checkbox', 'wio'),
                    'Edit a link' => esc_html__('Edit a link', 'wio'),
                    'Add a condition' => esc_html__('Add a condition', 'wio'),
                    'Total price' => esc_html__('Total price', 'wio'),
                    'Define as start step' => esc_html__('Define as start step', 'wio'),
                    'Link to another step' => esc_html__('Link to another step', 'wio'),
                    'Visibility conditions' => esc_html__('Visibility conditions', 'wio'),
                    'errorImport' => esc_html__('The uploaded file seems invalid', 'wio'),
                    'Add a new field' => esc_html__('Add a new field', 'wio'),
                    'Selectable image' => esc_html__('Selectable image', 'wio')
                )
            );
            wp_localize_script($this->parent->_token . '_admin', 'wio_data', $js_data);
        }
    }

    /**
     * Add menu to admin
     * @return void
     */
    public function add_menu_item() {
        if (
                in_array(
                        'woocommerce/woocommerce.php',
                        apply_filters('active_plugins', get_option('active_plugins'))
                )
        ) {
            add_menu_page('Infinite options', esc_html__('Infinite options', 'wutb'), 'manage_options', $this->parent->_token . '_menu', array($this, 'view_backend'), 'dashicons-lightbulb');
        }
    }

    public function previewCustomizer() {
        if (current_user_can('manage_options')) {
            if (isset($_POST['customizerID']) && isset($_POST['wooProduct'])) {
                $wooID = intval($_POST['wooProduct']);
                $customizerID = intval($_POST['customizerID']);
                $url = get_permalink($wooID);

                if (strpos($url, '?') !== false) {
                    $url .= '&wio_preview=' . $customizerID;
                } else {
                    $url .= '?wio_preview=' . $customizerID;
                }

                echo $url;
            }
            die();
        }
    }

    public function checkFirstStart() {
        if (current_user_can('manage_options')) {
            $settings = $this->getSettings();
            if (!$settings->firstStart) {
                global $wpdb;
                $table_name = $wpdb->prefix . "wio_settings";
                $wpdb->update($table_name, array('firstStart' => true), array('id' => 1));

                $this->updateCustomizersCSS();
            }
        }

//
    }

    public function view_backend() {
        if (
                in_array(
                        'woocommerce/woocommerce.php',
                        apply_filters('active_plugins', get_option('active_plugins'))
                )
        ) {
            if (current_user_can('manage_options')) {
                $settings = $this->getSettings();
                global $wpdb;

                echo '<div class="wio_root wio_bootstrap">'
                . '<div id="wio_loader"><div class="wio_ellipsis"><div></div><div></div><div></div><div></div></div></div>'
                . '<div id="wio_mainHeader" class="swatch-teal p-3">'
                . '<div class="float-end">'
                . '<a href="javascript:" data-wioaction="showWinLicense" class="btn btn-secondary btn-sm " ><i class="fas fa-user-lock me-2"></i>' . esc_html__('Purchase code', 'wio') . '</a>'
                . '<a data-wioaction="returnToList" class="btn btn-sm btn-circle" ><i class="fas fa-times"></i></a>'
                . '</div>'
                . '<div class="wio_logoCt"><i class="far fa-lightbulb me-2"></i><span>' . esc_html__('Woo Infinite Options', 'wio') . '</span></div>'
                . '</div>';
                echo '<div id="wio_content" class="">';
                include 'backend_templates/wio_customizersList.php';
                include 'backend_templates/wio_customizerView.php';
                echo '</div>';

                echo '
    <div id="wio_winLicense" class="modal " tabindex="-1">
        <div class="modal-dialog">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title">' . esc_html__('Your license', 'wio') . '</h5>
                    <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
                </div>
                <div class="modal-body">
                                       <div class="alert alert-info text-center">' . esc_html__('Filling your purchase code will activate the automatic updates', 'wio') . '</div>

                    <div class="form-group">     
                            <label for="wio_licensePurchaseField" class="mb-2">' . esc_html__('Purchase code', 'wio') . '</label>
                            <input id="wio_licensePurchaseField" name="purchaseCode" type="text" value="'.$settings->purchaseCode.'" class="form-control" />
                        </div>
                        <small><a href="https://www.loopus-plugins.com/purchaseCode.gif" target="_blank">' . esc_html__('Where can I find my purchase code ?', 'wio') . '</a></small>
                </div>
                <div class="modal-footer">
                    <a href="javascript:" data-wioaction="validatePurchaseCode" type="button" class="btn btn-primary"><i class="fas fa-check me-2"></i>' . esc_html__('Validate', 'wio') . '</a>

                </div>
            </div>
        </div>
    </div>';

                echo '</div>';
            }
        }
    }

    public function validatePurchaseCode() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            if (isset($_POST['code'])) {
                $code = sanitize_text_field($_POST['code']);
                try {

                    $url = 'https://www.loopus-plugins.com/updates/update.php?checkCode=32601790&code=' . $code;
                    $ch = curl_init($url);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    $rep = curl_exec($ch);
                    if ($rep != '0410') {
                        $table_name = $wpdb->prefix . "wio_settings";
                        $wpdb->update($table_name, array('purchaseCode' => $code), array('id' => 1));
                    } else {
                        $table_name = $wpdb->prefix . "wio_settings";
                        $wpdb->update($table_name, array('purchaseCode' => ''), array('id' => 1));
                        echo '1';
                    }
                } catch (Throwable $t) {
                    $table_name = $wpdb->prefix . "wio_settings";
                    $wpdb->update($table_name, array('purchaseCode' => $code), array('id' => 1));
                } catch (Exception $e) {
                    $table_name = $wpdb->prefix . "wio_settings";
                    $wpdb->update($table_name, array('purchaseCode' => $code), array('id' => 1));
                }
            }
            die();
        }
    }

    public function getSettings() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wio_settings";
        $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
        $rep = false;
        if (count($settings) > 0) {
            $rep = $settings[0];
        }
        return $rep;
    }

    public function loadCustomizers() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizers = $wpdb->get_results("SELECT id,jsonData FROM $table_name ORDER BY id ASC");
            foreach ($customizers as $customizer) {
                $customizer->jsonData = stripslashes($customizer->jsonData);                              
                
            }
            echo json_encode($customizers, true);
            die();
        }
    }

    public function createCustomizer() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            if (isset($_POST['customizerData'])) {
                $customizerData = ($_POST['customizerData']);
                $table_name = $wpdb->prefix . "wio_customizers";
                $wpdb->insert($table_name, array('jsonData' => $customizerData));
                echo $wpdb->insert_id;
                die();
            }
        }
    }

    public function saveCustomizer() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            if (isset($_POST['customizerData'])) {
                $customizerID = intval($_POST['customizerID']);
                $customizerData = ($_POST['customizerData']);
                $table_name = $wpdb->prefix . "wio_customizers";


                if (function_exists('icl_register_string')) {
                    $jsonData = json_decode(stripslashes($customizerData));
                    icl_register_string('wio', 'wio_' . $customizerID . '_resumeTitle', $jsonData->settings->resumeTitle);
                    icl_register_string('wio', $customizerID . '_resumeTitle', $jsonData->settings->resumeTitle);
                    icl_register_string('wio', $customizerID . '_txtButtonLabel', $jsonData->settings->txtButtonLabel);
                    icl_register_string('wio', $customizerID . '_txtNextStepLabel', $jsonData->settings->txtNextStepLabel);
                    icl_register_string('wio', $customizerID . '_txtOptionName', $jsonData->settings->txtOptionName);
                    icl_register_string('wio', $customizerID . '_txtTotal', $jsonData->settings->txtTotal);
                    icl_register_string('wio', $customizerID . '_txtValue', $jsonData->settings->txtValue);
                    icl_register_string('wio', $customizerID . '_txtPrice', $jsonData->settings->txtPrice);

                    foreach ($jsonData->steps as $step) {
                        icl_register_string('wio', $customizerID . '_step_' . $step->id, $step->title);

                        foreach ($step->items as $item) {
                            icl_register_string('wio', $customizerID . '_item_' . $item->id . '_label', $item->label);
                            icl_register_string('wio', $customizerID . '_item_' . $item->id . '_selectOptions', $item->selectOptions);
                            icl_register_string('wio', $customizerID . '_item_' . $item->id . '_text', $item->text);
                        }
                    }
                }

                $wpdb->update($table_name, array('jsonData' => $customizerData), array('id' => $customizerID));
                echo $customizerID;

                $this->updateCustomizersCSS();
                die();
            }
        }
    }

    public function removeCustomizer() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            if (isset($_POST['customizerID'])) {
                $customizerID = intval($_POST['customizerID']);

                if (function_exists('icl_register_string')) {
                    $customizers = $wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name WHERE id='%s' LIMIT 1", $customizerID));
                    if (count($customizers) > 0) {
                        $customizer = $customizers[0];
                        $jsonData = json_decode(stripslashes($customizer->jsonData));
                        icl_unregister_string('wio', $customizerID . '_resumeTitle');
                        icl_unregister_string('wio', $customizerID . '_txtButtonLabel');
                        icl_unregister_string('wio', $customizerID . '_txtNextStepLabel');
                        icl_unregister_string('wio', $customizerID . '_txtOptionName');
                        icl_unregister_string('wio', $customizerID . '_txtTotal');
                        icl_unregister_string('wio', $customizerID . '_txtValue');
                        icl_unregister_string('wio', $customizerID . '_txtPrice');


                        foreach ($jsonData->steps as $step) {
                            icl_unregister_string('wio', $customizerID . '_step_' . $step->id);

                            foreach ($step->items as $item) {
                                icl_unregister_string('wio', $customizerID . '_item_' . $item->id . '_label');
                                icl_unregister_string('wio', $customizerID . '_item_' . $item->id . '_selectOptions');
                                icl_unregister_string('wio', $customizerID . '_item_' . $item->id . '_text');
                            }
                        }
                    }
                }

                $wpdb->delete($wpdb->prefix . 'wio_customizers', array('id' => $customizerID));

                $this->updateCustomizersCSS();
            }
            die();
        }
    }

    public function duplicateCustomizer() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            $customizerID = intval($_POST['customizerID']);
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizers = $wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name WHERE id='%s' LIMIT 1", $customizerID));
            if (count($customizers) > 0) {
                $customizer = $customizers[0];
                $wpdb->insert($table_name, array('jsonData' => $customizer->jsonData));
                $newID = $wpdb->insert_id;

                if (function_exists('icl_register_string')) {
                    $jsonData = json_decode(stripslashes($customizer->jsonData));
                    icl_register_string('wio', $newID . '_resumeTitle', $jsonData->settings->resumeTitle);
                    icl_register_string('wio', $newID . '_txtButtonLabel', $jsonData->settings->txtButtonLabel);
                    icl_register_string('wio', $newID . '_txtNextStepLabel', $jsonData->settings->txtNextStepLabel);
                    icl_register_string('wio', $newID . '_txtOptionName', $jsonData->settings->txtOptionName);
                    icl_register_string('wio', $newID . '_txtTotal', $jsonData->settings->txtTotal);
                    icl_register_string('wio', $newID . '_txtValue', $jsonData->settings->txtValue);
                    icl_register_string('wio', $newID . '_txtPrice', $jsonData->settings->txtPrice);


                    foreach ($jsonData->steps as $step) {
                        icl_register_string('wio', $newID . '_step_' . $step->id, $step->title);

                        foreach ($step->items as $item) {
                            icl_register_string('wio', $newID . '_item_' . $item->id . '_label', $item->label);
                            icl_register_string('wio', $newID . '_item_' . $item->id . '_selectOptions', $item->selectOptions);
                            icl_register_string('wio', $newID . '_item_' . $item->id . '_text', $item->text);
                        }
                    }
                }
            }
            die();
        }
    }

    public function checkAutomaticUpdates() {
        if (current_user_can('manage_options')) {
            $settings = $this->getSettings();
            if ($settings && $settings->purchaseCode != "") {
                require 'kernl-update-checker/kernl-update-checker.php';
                $updateChecker = Puc_v4_Factory::buildUpdateChecker(
                                'https://kernl.us/api/v1/updates/60b0936cdac45d092188cac2/',
                                __FILE__,
                                'wio'
                );
                $updateChecker->purchaseCode = $settings->purchaseCode;
            }
        }
    }

    public function create_product_edition_tab($tabs) {
        $tabs['wio_customizer'] = array(
            'label' => esc_html__('Customization', 'wio'),
            'target' => 'wio_productPanel',
            'class' => array('wio_tab', 'show_if_simple', 'show_if_variable'),
            'priority' => 80
        );
        return $tabs;
    }

    public function display_product_edition_fields() {
        global $wpdb;
        global $post;

        $product = wc_get_product(get_the_ID());
        $current_product_id = 0;
        if ($product && !is_null($product)) {
            $current_product_id = $product->get_id();
        }
        $wooProduct = new WC_Product($current_product_id);

        echo'<div id="wio_productPanel" class="panel woocommerce_options_panel">';
        echo '      <div class="options_group">';

        $optionsArray = array();
        $optionsArray[''] = esc_html__('No customization', 'wio');
        $table_name = $wpdb->prefix . "wio_customizers";
        $customizers = $wpdb->get_results("SELECT id,jsonData FROM $table_name ORDER BY id ASC");
        foreach ($customizers as $customizer) {
            $jsonData = json_decode(stripslashes($customizer->jsonData));
            $optionsArray[$customizer->id] = $jsonData->title;
        }

        woocommerce_wp_select(array(
            'id' => 'wio_productOption_customizer',
            'label' => esc_html__('Customizer', 'wio'),
            'desc_tip' => true,
            'description' => esc_html__('Assign a customizer to this product', 'wio'),
            'options' => $optionsArray
        ));

        woocommerce_wp_checkbox(
                array(
                    'id' => 'wio_productOption_required',
                    'label' => esc_html__('Is mandatory', 'wio'),
                    'desc_tip' => true,
                    'description' => esc_html__('If this option is enabled, the customer will be forced to use the customizer to buy the product.', 'wio')
                )
        );

        $imgClass = 'btn-secondary';
        if ($product->get_meta('wio_productOption_dynamicImg') && $product->get_meta('wio_productOption_dynamicImg') == 1) {
            $imgClass = 'btn-primary';
        }

        echo '</div>';
        echo '</div>';
    }

    public function save_product_edition_fields($post_id) {
        $product = wc_get_product($post_id);

        if (isset($_POST['wio_productOption_customizer'])) {
            $product->update_meta_data('wio_productOption_customizer', intval($_POST['wio_productOption_customizer']));
        }

        $woocommerce_custom_product_checkbox = isset($_POST['wio_productOption_required']) ? 'yes' : 'no';
        $product->update_meta_data('wio_productOption_required', $woocommerce_custom_product_checkbox);

        $product->save();
    }

    private function updateCustomizersCSS() {

        if (current_user_can('manage_options')) {
            global $wpdb;
            $colorsStyles = '/* Custom styles for Woo Infinite Options */';
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizers = $wpdb->get_results("SELECT * FROM $table_name ORDER By id ASC");
            foreach ($customizers as $customizer) {
                $customizerID = $customizer->id;

                $jsonData = json_decode(stripslashes($customizer->jsonData));

                if ($jsonData->settings->googleFont != '') {
                    $fontname = $jsonData->settings->googleFont;
                    $fontname = str_replace(' ', '+', $fontname);
                    $colorsStyles .= '@import url(https://fonts.googleapis.com/css?family=' . $fontname . ':400,700);';

                    $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"]  {';
                    $colorsStyles .= ' font-family:"' . $jsonData->settings->googleFont . '"; ';
                    $colorsStyles .= '}';
                }
                $colorsStyles .= '#wio_productButton[data-wioid="' . $customizerID . '"] {'
                        . 'background-color: ' . $jsonData->settings->buttonColor . ';'
                        . 'color: ' . $jsonData->settings->buttonTxtColor . ';'
                        . '}';

                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .dropzone .dz-message .dz-button  {'
                        . 'color: ' . $jsonData->settings->textColor . ';'
                        . '}';
                $colorsStyles .= '#wio_productSelection[data-wioid="' . $customizerID . '"],'
                        . '#wio_productSelection[data-wioid="' . $customizerID . '"] table {'
                        . 'font-size: ' . $jsonData->settings->resumeFontSize . 'px;'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] {'
                        . 'font-size: ' . $jsonData->settings->fontSize . 'px;'
                        . 'padding: ' . $jsonData->settings->panelPadding . 'px;'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_fieldBloc .wio_tooltipIcon{'
                        . 'color: ' . $jsonData->settings->nextStepColor . ';'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .pretty label{'
                        . 'line-height: ' . (intval($jsonData->settings->fontSize)+2) . 'px;'
                        . 'font-size: ' . $jsonData->settings->fontSize . 'px;'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_buttonField {'
                        . 'background-color: ' . $jsonData->settings->otherBtnsColor . ';'
                        . 'color: ' . $jsonData->settings->otherBtnsTxtColor . ';'
                        . '}';

                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_imgCheckmark {'
                        . 'color: ' . $jsonData->settings->checkboxColor . ';'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"].wio_modal .wio_btnCloseModal  {'
                        . 'color: ' . $jsonData->settings->nextStepColor . ';'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] #wio_nextStepBtn {'
                        . 'background-color: ' . $jsonData->settings->nextStepColor . ';'
                        . 'color: ' . $jsonData->settings->nextStepTxtColor . ';'
                        . '}';

                $colorsStyles .= 'body #wio_customizer[data-wioid="' . $customizerID . '"] input:not([type="checkbox"]):not([type="radio"]) {'
                        . 'background-color: ' . $jsonData->settings->fieldsColor . ';'
                        . 'color: ' . $jsonData->settings->fieldsTxtColor . ';'
                        . 'border-color: ' . $jsonData->settings->fieldsBorderColor . ';'
                        . 'border-width: ' . $jsonData->settings->fieldsBorderWidth . 'px;'
                        . 'font-size: ' . $jsonData->settings->fontSize . 'px;'
                        . '}';

                $colorsStyles .= 'body #wio_customizer[data-wioid="' . $customizerID . '"] .pretty.p-switch input:checked~.state:before,'
                        . ' body #wio_customizer[data-wioid="' . $customizerID . '"] .pretty.p-switch .state:before {'
                        . 'border-color: ' . $jsonData->settings->fieldsBorderColor . ';'
                        . '}';


                $colorsStyles .= 'body #wio_customizer[data-wioid="' . $customizerID . '"] {'
                        . 'background-color: ' . $jsonData->settings->backgroundColor . ';'
                        . 'color: ' . $jsonData->settings->textColor . ';'
                        . '}';


                $colorsStyles .= '#wio_customizerMeta[data-wioid="' . $customizerID . '"] table th,'
                        . '#wio_customizerMeta[data-wioid="' . $customizerID . '"] table td {'
                        . 'border-color: ' . $jsonData->settings->tableBorderColor . ';'
                        . '}';


                $colorsStyles .= 'body #wio_customizer[data-wioid="' . $customizerID . '"] .pretty.p-switch input:checked~.state label:after,'
                        . 'body #wio_customizer[data-wioid="' . $customizerID . '"]  .pretty.p-default input:checked~.state label:after{'
                        . 'background-color: ' . $jsonData->settings->checkboxColor . ' !important;'
                        . '}';

                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_field.dz-clickable {'
                        . 'border-color: ' . $jsonData->settings->tableHeaderColor . ';'
                        . '}';

                $colorsStyles .= '#wio_customizerMeta[data-wioid="' . $customizerID . '"] table thead th {'
                        . 'background-color: ' . $jsonData->settings->tableHeaderColor . ';'
                        . 'color: ' . $jsonData->settings->tableHeaderTxtColor . ';'
                        . '}';

                $colorsStyles .= '#wio_customizerMeta[data-wioid="' . $customizerID . '"] tbody td {'
                        . 'background-color: ' . $jsonData->settings->tableBgColor . ';'
                        . 'color: ' . $jsonData->settings->tableTxtColor . ';'
                        . '}';


                $colorsStyles .= '#wio_wooproduct_optionsPrice[data-wioid="' . $customizerID . '"] {'
                        . 'color: ' . $jsonData->settings->buttonColor . ';'
                        . '}';
                $colorsStyles .= '#wio_customizerMeta[data-wioid="' . $customizerID . '"] .wio_button,'
                        . '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_button {'
                        . 'border-radius: ' . $jsonData->settings->btnsRadius . 'px;'
                        . '}';
                $colorsStyles .= '#wio_customizer[data-wioid="' . $customizerID . '"] .wio_invalid input:not([type="checkbox"]):not([type="radio"]):not([type="color"]) {'
                        . 'color: ' . $jsonData->settings->errorColor . ';'
                        . 'border-color: ' . $jsonData->settings->errorColor . ';'
                        . '}';
            }
        }
        chmod(plugin_dir_path(__FILE__) . '../assets/css/wio_customCss.css', $this->parent->chmodWrite);
        $fp = fopen(plugin_dir_path(__FILE__) . '../assets/css/wio_customCss.css', 'w');
        fwrite($fp, $colorsStyles);
        fclose($fp);
    }

    public function exportData() {
        if (current_user_can('manage_options')) {
            global $wpdb;
            if (!is_dir(plugin_dir_path(__FILE__) . '../tmp')) {
                mkdir(plugin_dir_path(__FILE__) . '../tmp');
                chmod(plugin_dir_path(__FILE__) . '../tmp', $this->parent->chmodWrite);
            }
            chmod(plugin_dir_path(__FILE__) . '../tmp', $this->parent->chmodWrite);

            $destination = plugin_dir_path(__FILE__) . '../tmp/wio_export.zip';
            if (file_exists($destination)) {
                unlink($destination);
            }
            $zip = new ZipArchive();
            if ($zip->open($destination, ZipArchive::CREATE) !== true) {
                return false;
            }

            $jsonExport = array();
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizers = array();
            foreach ($wpdb->get_results("SELECT * FROM $table_name") as $key => $value) {

                $jsonData = json_decode(stripslashes($value->jsonData));
                foreach ($jsonData->imgConditions as $img) {

                    $original_image = $img->image;
                    $upload_dir = wp_upload_dir();
                    $pos1 = strrpos($original_image, '/');
                    $pos2 = strrpos($img->image, '/', 0 - (strlen($img->image) - $pos1) - 1);
                    $pos3 = strrpos($img->image, '/', 0 - (strlen($img->image) - $pos2) - 1);
                    if (strpos($img->image, site_url()) !== false) {
                        $img->image = substr($img->image, strlen(site_url()) + 1);
                    }
                    if (strrpos($img->image, "wp-content") > -1) {
                        $img->image = substr($img->image, strrpos($img->image, "wp-content") + 11);
                    }
                    if (file_exists($this->dir . "/../../" . $img->image)) {
                        $zip->addfile($this->dir . "/../../" . $img->image, substr($original_image, $pos1 + 1));
                    }
                }
                $value->jsonData = ($jsonData);
                $customizers[] = $value;
            }
            $jsonExport['customizers'] = $customizers;

            $fp = fopen(plugin_dir_path(__FILE__) . '../tmp/wio_export.json', 'w');
            fwrite($fp, json_encode($jsonExport));
            fclose($fp);

            $zip->addfile(plugin_dir_path(__FILE__) . '../tmp/wio_export.json', 'wio_export.json');
            $zip->close();
            echo '1';
            die();
        }
    }

    public function checkActions() {
        global $wpdb;
        if (isset($_GET['wio_action']) && $_GET['wio_action'] == 'exportData') {
            $target_path = plugin_dir_path(__FILE__) . '../tmp/wio_export.zip';
            header('Content-type: application/zip');
            header('Content-Disposition: attachment; filename="' . basename($target_path) . '"');
            header("Content-Transfer-Encoding: Binary");
            header("Content-length: " . filesize($target_path));
            header("Pragma: no-cache");
            header("Expires: 0");
            ob_clean();
            flush();
            readfile($target_path);
            unlink($target_path);
            exit;
        }
    }

    public function importData() {
        if (current_user_can('manage_options')) {
            global $wpdb;

            if (isset($_FILES['importFile'])) {
                $error = false;
                if (!is_dir(plugin_dir_path(__FILE__) . '../tmp')) {
                    mkdir(plugin_dir_path(__FILE__) . '../tmp');
                    chmod(plugin_dir_path(__FILE__) . '../tmp', $this->parent->chmodWrite);
                }
                $target_path = plugin_dir_path(__FILE__) . '../tmp/wio_export.zip';
                if (@move_uploaded_file($_FILES['importFile']['tmp_name'], $target_path)) {


                    $upload_dir = wp_upload_dir();
                    if (!is_dir($upload_dir['path'])) {
                        mkdir($upload_dir['path']);
                    }

                    $zip = new ZipArchive;
                    $res = $zip->open($target_path);
                    if ($res === TRUE) {
                        $zip->extractTo(plugin_dir_path(__FILE__) . '../tmp/');
                        $zip->close();

                        $formsData = array();

                        $jsonfilename = 'wio_export.json';
                        $file = file_get_contents(plugin_dir_path(__FILE__) . '../tmp/' . $jsonfilename);
                        $dataJson = json_decode($file, true);

                        $table_name = $wpdb->prefix . "wio_customizers";
                        $wpdb->query("TRUNCATE TABLE $table_name");
                        if (array_key_exists('customizers', $dataJson)) {
                            foreach ($dataJson['customizers'] as $key => $value) {

                                $jsonData = (($value['jsonData']));
                                $i = 0;
                                foreach ($jsonData['imgConditions'] as $img) {
                                    $img_name = substr($img['image'], strrpos($img['image'], '/') + 1);
                                    $imagePath = substr($img['image'], 0, strrpos($img['image'], '/'));
                                    if (!file_exists(site_url() . '/' . $img['image'])) {
                                        if (!is_dir($imagePath)) {
                                            $imagePath = wp_upload_dir();
                                        }
                                        if (strrpos($img['image'], "uploads") === false) {
                                            $img['image'] = 'uploads' . $img['image'];
                                        }
                                        if (is_file(plugin_dir_path(__FILE__) . '../tmp/' . $img_name)) {
                                            copy(plugin_dir_path(__FILE__) . '../tmp/' . $img_name, $imagePath['basedir'] . $imagePath['subdir'] . '/' . $img_name);
                                        }
                                    }
                                    $img['image'] = $imagePath['url'] . '/' . $img_name;
                                    $jsonData['imgConditions'][$i]['image'] = $imagePath['url'] . '/' . $img_name;
                                    $i++;
                                }
                                $value['jsonData'] = addslashes(json_encode($jsonData));

                                $wpdb->insert($table_name, $value);
                            }
                        }

                        $files = glob(plugin_dir_path(__FILE__) . '../tmp/*');
                        foreach ($files as $file) {
                            if (is_file($file))
                                unlink($file);
                        }
                    } else {
                        $error = true;
                    }
                } else {
                    $error = true;
                }
                if ($error) {
                    echo esc_html__('An error occurred during the transfer', 'wio');
                    die();
                } else {
                    echo 1;
                    die();
                }
            }
        }
    }

    /**
     * Main Instance
     *
     *
     * @since 1.0.0
     * @static
     * @return Main instance
     */
    public static function instance($parent) {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($parent);
        }
        return self::$_instance;
    }

// End instance()

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone() {
        _doing_it_wrong(__FUNCTION__, '', $this->parent->_version);
    }

// End __clone()

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup() {
        
    }

// End __wakeup()
}
