<?php

if (!defined('ABSPATH'))
    exit;

class wio_core {

    /**
     * The single instance
     * @var    object
     * @access  private
     * @since    1.0.0
     */
    private static $_instance = null;

    /**
     * Settings class object
     * @var     object
     * @access  public
     * @since   1.0.0
     */
    public $settings = null;

    /**
     * The version number.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_version;

    /**
     * The token.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_token;

    /**
     * The main plugin file.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $file;

    /**
     * The main plugin directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $dir;

    /**
     * The plugin assets directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_dir;

    /**
     * The plugin assets URL.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_url;

    /**
     * Suffix for Javascripts.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $templates_url;

    /**
     * Suffix for Javascripts.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $script_suffix;

    /**
     * For menu instance
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $menu;

    /**
     * For template
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $plugin_slug;

    /**
     * Current customizer
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $currentCustomizer = 0;

    /**
     * Constructor function.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function __construct($file = '', $version = '1.0.0') {
        $this->_version = $version;
        $this->_token = 'wio';
        $this->file = $file;
        $this->dir = dirname($this->file);
        $this->assets_dir = trailingslashit($this->dir) . 'assets';
        $this->assets_url = esc_url(trailingslashit(plugins_url('/assets/', $this->file)));

        add_action('wp_enqueue_scripts', array($this, 'load_frontend_scripts'), 10, 1);
        add_action('wp_enqueue_scripts', array($this, 'load_frontend_styles'), 10, 1);

        add_action('wp_ajax_nopriv_wio_finishCustomization', array($this, 'finishCustomization'));
        add_action('wp_ajax_wio_finishCustomization', array($this, 'finishCustomization'));
        add_action('wp_ajax_nopriv_wio_uploadFiles', array($this, 'uploadFiles'));
        add_action('wp_ajax_wio_uploadFiles', array($this, 'uploadFiles'));
        add_filter('woocommerce_add_cart_item_data', array($this, 'customDataToWooCart'), 99, 2);
        add_filter('woocommerce_cart_item_name', array($this, 'renderCartProductData'), 99, 3);
        add_action('woocommerce_before_calculate_totals', array($this, 'wooCommerceCalculateTotal'), 99);
        add_filter('woocommerce_loop_add_to_cart_link', array($this, 'replaceAddCartButton'), 99, 3);
        add_action('woocommerce_add_order_item_meta', array($this, 'addSelectionToOrder'), 99, 2);
        add_action('woocommerce_cart_item_thumbnail', array($this, 'replaceCartProductImage'), 99, 2);
        add_action('plugins_loaded', array($this, 'init_localization'));


        $this->chmodWrite = ( 0747 & ~ umask() );
        if (defined('FS_CHMOD_DIR')) {
            $this->chmodWrite = FS_CHMOD_DIR;
        }
        if ($this->chmodWrite == 0745) {
            $this->chmodWrite = 0755;
        }
        $upload_dir = wp_upload_dir();
        if (!is_dir($upload_dir['basedir'] . '/wio')) {
            mkdir($upload_dir['basedir'] . '/wio');
            chmod($upload_dir['basedir'] . '/wio', $this->chmodWrite);
        }
        $this->uploads_dir = $upload_dir['basedir'] . '/wio/';
        $this->uploads_url = $upload_dir['baseurl'] . '/wio/';
    }

    /*
     * Plugin init localization
     */

    public function init_localization() {
        $moFiles = scandir(trailingslashit($this->dir) . 'languages/');
        foreach ($moFiles as $moFile) {
            if (strlen($moFile) > 3 && substr($moFile, -3) == '.mo' && strpos($moFile, get_locale()) > -1) {
                load_textdomain('wio', trailingslashit($this->dir) . 'languages/' . $moFile);
            }
        }
    }

    public function replaceCartProductImage($product_img, $cart_item) {

        $class = 'attachment-shop_thumbnail wp-post-image';
        if (isset($cart_item['wio_image']) && $cart_item['wio_image'] !== null && $cart_item['wio_image'] != '') {
            $src = $cart_item['wio_image'];
            $a = '<img src="' . $src . '" class="' . $class . '" />';
            return $a;
        } else {
            return $product_img;
        }
    }

    public function addSelectionToOrder($item_id, $values) {
        global $woocommerce, $wpdb;
        $customValue = $values['wio_customInfos'];
        if (!empty($customValue)) {
            $selection = json_decode(stripslashes($customValue), true);

            $render = "<table class='wdm_options_table wio_optionsTable'>";
            $checkedSteps = array();
            foreach ($selection as $item) {
                $stepColSpan = 2;
                if (isset($item['priceTxt'])) {
                    $stepColSpan = 3;
                }
                if (!in_array($item['step'], $checkedSteps)) {
                    $checkedSteps[] = $item['step'];
                    $render .= '<tr><td colspan="' . $stepColSpan . '" class="wio_optionTableStepTitle">' . $item['step'] . '</td></tr>';
                }

                $value = '';
                $dbPoints = '';
                if (array_key_exists('value', $item) && strlen($item['value']) > 0) {
                    $value = $item['value'];
                    $dbPoints = ' :';
                }
                $tdPrice = '';
                if (isset($item['priceTxt'])) {
                    $tdPrice = '<td class="wio_priceTd">' . $item['priceTxt'] . '</td>';
                }
                $render .= '<tr><td>' . $item['label'] . $dbPoints . ' </td><td class="wio_valueTd">' . $value . '</td>' . $tdPrice . '</tr>';
            }
            $render .= "</table>";

            wc_add_order_item_meta($item_id, esc_html__('Options', 'wio'), $render);
        }
    }

    public function replaceAddCartButton($button, $product) {
        $wooProduct = $product;
        $chkCustomBtn = false;
        $button_text = '';
        if ($wooProduct->exists()) {
            $customizerIDOption = get_post_meta($product->get_id(), 'wio_productOption_customizer');
            $required = get_post_meta($product->get_id(), 'wio_productOption_required');
            if ($required && ($required === true || $required[0] == 'yes') && $customizerIDOption && count($customizerIDOption) > 0) {
                global $wpdb;
                $customizerID = intval($customizerIDOption[0]);
                $table_name = $wpdb->prefix . "wio_customizers";
                $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $customizerID));
                if (count($customizer) > 0) {
                    $customizer = $customizer[0];
                    $chkCustomBtn = true;
                    $jsonData = json_decode(stripslashes($customizer->jsonData));
                    $button_text = $jsonData->settings->txtButtonLabel;
                }
            }
        }

        if (!$chkCustomBtn)
            return $button;


        return '<a class="button" href="' . $product->get_permalink() . '">' . $button_text . '</a>';
    }

    public function wooCommerceCalculateTotal($cart_object) {
        if (!WC()->session->__isset("reload_checkout")) {
            foreach ($cart_object->cart_contents as $key => $value) {

                if (isset($value["wio_price"])) {
                    $value['data']->set_price(floatval($value["wio_price"]));
                }
            }
        }
    }

    public function uploadFiles() {
        global $wpdb;
        $customizerID = intval($_POST['customizerID']);
        $fieldID = intval($_POST['fieldID']);
        $stepID = sanitize_text_field($_POST['stepID']);

        if (get_option('wio_' . $this->get_client_ip() . '_session')) {
            $sessionToken = get_option('wio_' . $this->get_client_ip() . '_session');
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $customizerID));
            if (count($customizer) > 0) {
                $customizer = $customizer[0];
                $jsonData = json_decode(stripslashes($customizer->jsonData));
                $step = $this->getCustomizerStep($jsonData, $stepID);
                if ($step) {
                    $field = $this->getFieldFromStep($step, $fieldID);
                    if ($field) {
                        $maxSize = intval($field->maxWeight);
                        if ($maxSize == 0) {
                            $maxSize = 5;
                        }
                        $maxSize = $maxSize * pow(1024, 2);
                        foreach ($_FILES as $key => $value) {
                            if ($value["error"] > 0) {
                                echo "error";
                            } else {
                                if (strlen($value["name"]) > 4 &&
                                        $value['size'] < $maxSize &&
                                        strpos(strtolower($value["name"]), '.php') === false &&
                                        strpos(strtolower($value["name"]), '.js') === false &&
                                        strpos(strtolower($value["name"]), '.html') === false &&
                                        strpos(strtolower($value["name"]), '.phtml') === false &&
                                        strpos(strtolower($value["name"]), '.pl') === false &&
                                        strpos(strtolower($value["name"]), '.py') === false &&
                                        strpos(strtolower($value["name"]), '.jsp') === false &&
                                        strpos(strtolower($value["name"]), '.asp') === false &&
                                        strpos(strtolower($value["name"]), '.htm') === false &&
                                        strpos(strtolower($value["name"]), '.shtml') === false &&
                                        strpos(strtolower($value["name"]), '.sh') === false &&
                                        strpos(strtolower($value["name"]), '.cgi') === false &&
                                        strpos(strtolower($value["name"]), '.htaccess') === false &&
                                        strpos(strtolower($value["name"]), '..') === false &&
                                        strpos(strtolower($value["name"]), '/') === false
                                ) {
                                    $fileName = str_replace('..', '', $value["name"]);
                                    $fileName = str_replace('/', '', $fileName);
                                    $fileName = str_replace(' ', '_', $fileName);
                                    $fileName = str_replace("'", '_', $fileName);
                                    $fileName = str_replace('"', '_', $fileName);
                                    $fileName = preg_replace('/[^a-zA-Z0-9._-]/', '', $fileName);

                                    $ext = $this->get_extension($value["name"]);
                                    if ($field->allowedExtensions == 'image/*') {
                                        $allowedFiles = array('.jpg', '.jpeg', '.png', '.gif');
                                    } else {
                                        $allowedFiles = explode(",", $field->allowedExtensions);
                                    }
                                    if (in_array('.' . strtolower($ext), $allowedFiles) || $field->allowedExtensions = '') {
                                        $randomF = $this->generateRandomString(5);
                                        $randomS = rand(1, 10000);
                                        if (!is_dir($this->uploads_dir . $sessionToken . $randomF)) {
                                            mkdir($this->uploads_dir . $sessionToken . $randomF);
                                            chmod($this->uploads_dir . $sessionToken . $randomF, $this->chmodWrite);
                                        }
                                        move_uploaded_file($value["tmp_name"], $this->uploads_dir . $sessionToken . $randomF . '/' . $randomS . '_' . $fileName);
                                        chmod($this->uploads_dir . $sessionToken . $randomF . '/' . $randomS . '_' . $fileName, 0644);
                                        echo $this->wp_upload_dir_url() . '/wio/' . $sessionToken . $randomF . '/' . $randomS . '_' . $fileName;
                                        die();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private function wp_upload_dir_url() {
        $upload_dir = wp_upload_dir();
        $upload_dir = $upload_dir['baseurl'];
        return preg_replace('/^https?:/', '', $upload_dir);
    }

    private function generateRandomString($length = 10) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    private function getCustomizerStep($jsonData, $stepID) {
        $rep = false;
        foreach ($jsonData->steps as $step) {
            if ($step->id == $stepID) {
                $rep = $step;
                break;
            }
        }
        return $rep;
    }

    private function get_extension($file) {
        $tmp = explode('.', $file);
        $extension = end($tmp);
        return $extension ? $extension : false;
    }

    private function getFieldFromStep($step, $itemID) {
        $rep = false;
        foreach ($step->items as $item) {
            if ($item->id == $itemID) {
                $rep = $item;
                break;
            }
        }
        return $rep;
    }

    public function renderCartProductData($product_name, $values, $cart_item_key) {
        $render = $product_name;

        if (array_key_exists('wio_customInfos', $values)) {
            $selection = $values['wio_customInfos'];
            if(!empty($selection) && is_array($selection)){
            $selection = json_decode(stripslashes($selection), true);

            $render = $product_name . "</a><dl class='variation'>";
            $render .= "<table class='wdm_options_table wio_optionsTable' id='" . $values['product_id'] . "'>";
            $checkedSteps = array();
            foreach ($selection as $item) {
                $stepColSpan = 2;
                if (isset($item['priceTxt'])) {
                    $stepColSpan = 3;
                }
                if (!in_array($item['step'], $checkedSteps)) {
                    $checkedSteps[] = $item['step'];
                    $render .= '<tr><td colspan="' . $stepColSpan . '" class="wio_optionTableStepTitle">' . $item['step'] . '</td></tr>';
                }

                $value = '';
                $dbPoints = '';
                if (array_key_exists('value', $item) && strlen($item['value']) > 0) {
                    $value = $item['value'];
                    $dbPoints = ' :';
                }
                $tdPrice = '';
                if (isset($item['priceTxt'])) {
                    $tdPrice = '<td class="wio_priceTd">' . $item['priceTxt'] . '</td>';
                }
                $render .= '<tr><td>' . $item['label'] . $dbPoints . ' </td><td class="wio_valueTd">' . $value . '</td>' . $tdPrice . '</tr>';
            }
            $render .= "</table></dl>";
        }
        }
        return $render;
    }

    public function load_frontend_scripts($hook = '') {
        global $product;

        if (is_cart() || is_checkout()) {
            $mustLoadCss = true;
            wp_register_script($this->_token . '_optionsTable', esc_url($this->assets_url) . 'js/wio_optionsTable.min.js', array('jquery'), $this->_version);
            wp_enqueue_script($this->_token . '_optionsTable');
        }

        if (!is_object($product))
            $product = wc_get_product(get_the_ID());
        $current_product_id = 0;
        if ($product && !is_null($product)) {
            $current_product_id = $product->get_id();
        }
        $wooProduct = new WC_Product($current_product_id);
        if ($wooProduct->exists()) {
            $customizerIDOption = get_post_meta($current_product_id, 'wio_productOption_customizer');
            $required = get_post_meta($current_product_id, 'wio_productOption_required');
            if (count($required) > 0 && $required[0] == 'yes') {
                $required = 1;
            } else {
                $required = 0;
            }
            if (isset($_GET['wio_preview']) || ($customizerIDOption && count($customizerIDOption) > 0)) {
                global $wpdb;
                if (isset($_GET['wio_preview'])) {
                    $customizerID = ($_GET['wio_preview']);
                } else {
                    $customizerID = intval($customizerIDOption[0]);
                }


                $table_name = $wpdb->prefix . "wio_customizers";
                $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $customizerID));
                if (count($customizer) > 0) {
                    $customizer = $customizer[0];
                    $this->currentCustomizer = $customizer->id;
                    $jsonData = json_decode(stripslashes($customizer->jsonData));
                    $jsonData->id = $customizer->id;
                    $jsonData->productID = $current_product_id;
                    $jsonData->currency = get_woocommerce_currency_symbol();
                    $jsonData->currencyPosition = get_option('woocommerce_currency_pos');

                    $jsonData->productPrice = $wooProduct->get_price();
                    $jsonData->productImage = '';
                    $jsonData->required = $required;
                    $jsonData->priceFormat = get_woocommerce_price_format();
                    $jsonData->thousandSeparator = wc_get_price_thousand_separator();
                    $jsonData->decimalSeparator = wc_get_price_decimal_separator();
                    $jsonData->decimalNb = wc_get_price_decimals();
                    
                    
                    $img = wp_get_attachment_image_src(get_post_thumbnail_id($current_product_id), 'single-post-thumbnail');
                    if (count($img) > 0) {
                        $jsonData->productImage = $img[0];
                    }
                    if (isset($jsonData->settings->useWPML) && $jsonData->settings->useWPML) {
                        if (function_exists('icl_register_string')) {
                            $customizerID = $customizer->id;
                            $jsonData->settings->resumeTitle = icl_t('wio', $customizerID . '_' . 'resumeTitle');
                            $jsonData->settings->txtButtonLabel = icl_t('wio', $customizerID . '_' . 'txtButtonLabel');
                            $jsonData->settings->txtButtonLabel = icl_t('wio', $customizerID . '_' . 'txtButtonLabel');
                            $jsonData->settings->txtNextStepLabel = icl_t('wio', $customizerID . '_' . 'txtNextStepLabel');
                            $jsonData->settings->txtOptionName = icl_t('wio', $customizerID . '_' . 'txtOptionName');
                            $jsonData->settings->txtTotal = icl_t('wio', $customizerID . '_' . 'txtTotal');
                            $jsonData->settings->txtValue = icl_t('wio', $customizerID . '_' . 'txtValue');
                            $jsonData->settings->txtPrice = icl_t('wio', $customizerID . '_' . 'txtPrice');

                            foreach ($jsonData->steps as $step) {
                                $jsonData->settings->txtPrice = icl_t('wio', $customizerID . '_step_' . $step->id);
                                foreach ($step->items as $item) {
                                    $item->label = icl_t('wio', $customizerID . '_item_' . $item->id . '_label');
                                    $item->selectOptions = icl_t('wio', $customizerID . '_item_' . $item->id . '_selectOptions');
                                    $item->text = icl_t('wio', $customizerID . '_item_' . $item->id . '_text');
                                }
                            }
                        }
                    }
                    if (isset($_GET['wio_preview'])) {
                        $jsonData->preview = 1;
                    } else {
                        $jsonData->preview = 0;
                    }
                    wp_register_script('dropzone', esc_url($this->assets_url) . 'js/dropzone.min.js', array('jquery'), $this->_version);
                    wp_enqueue_script('dropzone');

                    wp_register_script('anime', esc_url($this->assets_url) . 'js/anime.min.js', array(), $this->_version, true);
                    wp_enqueue_script('anime');
                    wp_register_script('jquery-waves', esc_url($this->assets_url) . 'js/jquery-waves.min.js', array('anime'), $this->_version, true);
                    wp_enqueue_script('jquery-waves');

                    wp_register_script('pickr', esc_url($this->assets_url) . 'js/pickr.min.js', array(), $this->_version, true);
                    wp_enqueue_script('pickr');

                    wp_register_script($this->_token . '_frontend', esc_url($this->assets_url) . 'js/wio_frontend.min.js', array('jquery'), $this->_version);
                    wp_enqueue_script($this->_token . '_frontend');


                    $jsonData->ajaxurl = admin_url('admin-ajax.php');
                    $js_data = (array) $jsonData;
                    wp_localize_script($this->_token . '_frontend', 'wio_data', $js_data);
                    update_option('wio_' . $this->get_client_ip() . '_session', md5(uniqid(rand(), true)));


                    add_action('wp_head', array($this, 'options_custom_styles'));
                }
            }
        }
    }

    public function options_custom_styles() {

        if ($this->currentCustomizer > 0) {

            global $wpdb;
            $outputCSS = '';
            $outputJS = '';
            
            $table_name = $wpdb->prefix . "wio_customizers";
            $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $this->currentCustomizer));
            if (count($customizer) > 0) {
                $customizer = $customizer[0];
                $jsonData = json_decode(stripslashes($customizer->jsonData));

                if (isset($jsonData->settings->customJS)&& $jsonData->settings->customJS != "") {
                    $outputJS = "\n<script>\n" . $jsonData->settings->customJS . "</script>\n";
                    echo $outputJS;
                }
                if (isset($jsonData->settings->customCSS)&& $jsonData->settings->customCSS != "") {
                    $outputCSS = "\n<style>\n" . $jsonData->settings->customCSS . "</style>\n";
                    echo $outputCSS;
                }
            }
        }
    }

    public function load_frontend_styles($hook = '') {
        global $product;
        $mustLoadCss = false;
        if (is_cart()) {
            $mustLoadCss = true;
        } else if (is_checkout()) {
            $mustLoadCss = true;
        } else {
            if (!is_object($product))
                $product = wc_get_product(get_the_ID());
            $current_product_id = 0;
            if ($product && !is_null($product)) {
                $current_product_id = $product->get_id();
            }
            $wooProduct = new WC_Product($current_product_id);
            if ($wooProduct->exists()) {
                $mustLoadCss = true;
            }
        }
        if ($mustLoadCss) {
            wp_register_style('fontawesome', esc_url($this->assets_url) . 'css/fontawesome_all.min.css', array(), $this->_version);
            wp_enqueue_style('fontawesome');
            wp_register_style('dropzone', esc_url($this->assets_url) . 'css/dropzone.min.css', array(), $this->_version);
            wp_enqueue_style('dropzone');
            wp_register_style('jquery-waves', esc_url($this->assets_url) . 'css/jquery-waves.min.css', array(), $this->_version);
            wp_enqueue_style('jquery-waves');

            wp_register_style($this->_token . '_pretty-checkbox', esc_url($this->assets_url) . 'css/wio_pretty-checkbox.min.css', array(), $this->_version);
            wp_enqueue_style($this->_token . '_pretty-checkbox');
            wp_register_style('pickr', esc_url($this->assets_url) . 'css/pickr_nano.min.css', array(), $this->_version);
            wp_enqueue_style('pickr');


            wp_register_style($this->_token . '_frontend', esc_url($this->assets_url) . 'css/wio_frontend.min.css', array(), $this->_version);
            wp_enqueue_style($this->_token . '_frontend');
            wp_register_style($this->_token . '_customCSS', esc_url($this->assets_url) . 'css/wio_customCss.css', array(), time());
            wp_enqueue_style($this->_token . '_customCSS');
        }
    }

    public function add_product_button() {
        global $wpdb;
        $current_product_id = get_the_ID();
        $product = wc_get_product($current_product_id);

        if ($product->is_type('simple') || $product->is_type('variable')) {
            $customizerIDOption = get_post_meta($current_product_id, 'wio_productOption_customizer');
            if ($customizerIDOption && count($customizerIDOption) > 0) {
                $customizerID = intval($customizerIDOption[0]);

                $table_name = $wpdb->prefix . "wio_customizers";
                $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $customizerID));
                if (count($customizer) > 0) {
                    $customizer = $customizer[0];
                    $jsonData = json_decode(stripslashes($customizer->jsonData));
                }
            }
        }
    }

    private function get_client_ip() {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP'))
            $ipaddress = getenv('HTTP_CLIENT_IP');
        else if (getenv('HTTP_X_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        else if (getenv('HTTP_X_FORWARDED'))
            $ipaddress = getenv('HTTP_X_FORWARDED');
        else if (getenv('HTTP_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        else if (getenv('HTTP_FORWARDED'))
            $ipaddress = getenv('HTTP_FORWARDED');
        else if (getenv('REMOTE_ADDR'))
            $ipaddress = getenv('REMOTE_ADDR');
        else
            $ipaddress = 'UNKNOWN';
        return $ipaddress;
    }

    public function finishCustomization() {
        if (isset($_POST['selection']) && isset($_POST['customizerID']) && isset($_POST['productID'])) {
            $customizerID = intval($_POST['customizerID']);
            $productID = intval($_POST['productID']);
            if (isset($_POST['image'])) {
                $image = sanitize_text_field($_POST['image']);
            } else {
                $image = '';
            }

            global $wpdb;

            $table_name = $wpdb->prefix . "wio_customizers";
            $customizer = $wpdb->get_results($wpdb->prepare("SELECT id,jsonData FROM $table_name WHERE id=%s LIMIT 1", $customizerID));
            if (count($customizer) > 0) {
                $customizer = $customizer[0];
                $jsonData = json_decode(stripslashes($customizer->jsonData));

                $selection = ($_POST['selection']);
                if ($jsonData->settings->modifyPrice) {

                    $wooProduct = new WC_Product($productID);
                    if ($wooProduct->exists()) {
                        $customizerIDOption = get_post_meta($productID, 'wio_productOption_customizer');
                        $chkPreview = false;

                        if (isset($_POST['preview']) && intval($_POST['preview']) == 1 && current_user_can('manage_options')) {
                            $chkPreview = true;
                        }
                        if ($chkPreview || ($customizerIDOption && count($customizerIDOption) > 0 && $customizerIDOption[0] == $customizerID)) {
                            $price = floatval($_POST['price']);
                            $price = filter_var($price, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
                            update_option('wio_' . $this->get_client_ip() . '_' . $productID . '_price', $price);
                        }
                    }
                }

                update_option('wio_' . $this->get_client_ip() . '_' . $productID, $selection);
                update_option('wio_' . $this->get_client_ip() . '_' . $productID . '_image', $image);
            }
        }
        die();
    }

    public function customDataToWooCart($cart_item_data, $product_id) {
        global $woocommerce;
        if (get_option('wio_' . $this->get_client_ip() . '_' . $product_id)) {
            $selection = get_option('wio_' . $this->get_client_ip() . '_' . $product_id);
            $price = get_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_price');

            $cart_item_data['wio_customInfos'] = $selection;
            $cart_item_data['wio_price'] = $price;
            $cart_item_data['wio_image'] = '';
            if (get_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_image') !== null && get_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_image')) {
                $cart_item_data['wio_image'] = get_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_image');
            }
            delete_option('wio_' . $this->get_client_ip() . '_' . $product_id);
            delete_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_price');
            delete_option('wio_' . $this->get_client_ip() . '_' . $product_id . '_image');
        }
        return $cart_item_data;
    }

    /**
     * Log the plugin version number.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    private function _log_version_number() {
        update_option($this->_token . '_version', $this->_version);
    }

    /**
     * Main wutb_Core Instance
     *
     *
     * @since 1.0.0
     * @static
     * @see wutb_Core()
     * @return Main wutb_Core instance
     */
    public static function instance($file = '', $version = '1.0.0') {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($file, $version);
        }
        return self::$_instance;
    }

// End instance()

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone() {
        
    }

// End __clone()

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup() {
        
    }

}

?>